package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.product.dto.CourseDTO;
import cn.rankin.data.api.product.dto.PackageDTO;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.data.api.product.dto.TrainingDTO;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.entity.Package;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.PackageProductRelationRepository;
import cn.rankin.productservice.repository.ProductRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
@Slf4j
public class ProductService {

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private CourseService courseService;

    @Autowired
    private SupportService supportService;

    @Autowired
    private PackageService packageService;

    @Autowired
    private TrainingService trainingService;

    @Autowired
    private PackageProductRelationRepository packageProductRelationRepository;

    public APIResult<Page<Product>> search(Product product, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = productRepository.count(product);
        Page<Product> page = new Page<>(count, pageNo, pageSize);

        Long start = page.getStart();
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Product> productList = productRepository.find(product, start, page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(productList);

        return APIResult.ok(page);
    }

    public APIResult get(String productId) {
        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        ProductTypeEnum type = product.getType();
        if (ProductTypeEnum.COURSE.equals(type)){
            return courseService.getCourse(productId);
        }else if (ProductTypeEnum.SUPPORT.equals(type)) {
            return supportService.getSupport(productId);
        }else if (ProductTypeEnum.PACKAGE.equals(type)) {
            return packageService.getPackage(productId);
        }else if (ProductTypeEnum.TRAINING.equals(type)) {
            Training entity =  trainingService.get(productId);
            if(null == entity){
                return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
            }
            Map<String, Object> ret = BeanUtil.convertToMap(entity, true);
            ret.put("type", ProductTypeEnum.TRAINING);
            return APIResult.ok(ret);
        }


        return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
    }


    public Map<String, Object> getProductDict(String productId) {
        Map<String, Object> result = new HashMap<>();


        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return result;
        }

        ProductTypeEnum type = product.getType();
        if (ProductTypeEnum.COURSE.equals(type)){
            Course entity = courseService.findCourseById(productId);
            if(null == entity){
                return null;
            }
            result = BeanUtil.convertToMap(entity, true);
            result.put("type", ProductTypeEnum.COURSE);
            result.put("name", result.get("title") +" | "+result.get("subTitle"));
        }else if (ProductTypeEnum.SUPPORT.equals(type)) {
            Support entity = supportService.findSupportById(productId);
            if(null == entity){
                return null;
            }
            result = BeanUtil.convertToMap(entity, true);
            result.put("type", ProductTypeEnum.SUPPORT);
            result.put("name", result.get("title") +" | "+result.get("subTitle"));
        }else if (ProductTypeEnum.PACKAGE.equals(type)) {
            Package entity = packageService.findPackageById(productId);
            if(null == entity){
                return null;
            }
            result = BeanUtil.convertToMap(entity, true);
            result.put("type", ProductTypeEnum.PACKAGE);
        }else if (ProductTypeEnum.TRAINING.equals(type)) {
            Training entity =  trainingService.get(productId);
            if(null == entity){
                return null;
            }
            result = BeanUtil.convertToMap(entity, true);
            result.put("type", ProductTypeEnum.TRAINING);
            result.put("name", result.get("title"));

        }

        return result;
    }

    public APIResult<Package> getPackage(String productId) {
        return packageService.getPackage(productId);
    }

    /**
     * Get Related Packge by pid
     * @param pid
     * @param start
     * @param offset
     * @param sortKey
     * @param direction
     * @return Plain Product, without extra fields like childrens etc.
     */
    public List<Product> getRelatedPackages(String pid, String merchantId, Long start, Integer offset, String sortKey, Sort.Direction direction){
        List<PackageProductRelation> currentRelationList = packageProductRelationRepository.findByPid(pid);

        //get pkgs
        List<String> pids = new ArrayList<>();

        for(PackageProductRelation rel : currentRelationList){
            //skip pid not belong merchant
            String pkgId = rel.getPkgId();
            if(null == pkgId || null == merchantProductRepository.findByPidAndMerchantIdAndStatus(pkgId, merchantId, BaseStatusEnum.NORMAL)){
                log.info("Skip, pkgId not Belong Merchant Or Not on Sale, pkgId={}, merchantId={}", pkgId, merchantId);
                continue;
            }

            pids.add(pkgId);
        }

        if (pids.isEmpty()){
            log.info("No Related Package Find By pid, pid={}", pid);
            return new ArrayList<>();
        }

        List<Product> pkgs = productRepository.findByPidsAndStatus(pids, BaseStatusEnum.NORMAL);

        if (null == pkgs){
            pkgs = new ArrayList<>();
        }
        return pkgs;
    }

    /**
     *
     * @param pid
     * @return
     */
    public Product getProduct(String pid){
        return productRepository.findByPid(pid);
    }


    @Transactional
    public APIResult delete(String productId) {
        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        productRepository.deleteByPid(productId);

        ProductTypeEnum type = product.getType();
        if (type.equals(ProductTypeEnum.COURSE)) {
            courseService.delete(productId);
        }else if (type.equals(ProductTypeEnum.SUPPORT)) {
            supportService.delete(productId);
        }else if (type.equals(ProductTypeEnum.PACKAGE)){
            packageService.deletePackage(productId);
        }else if (type.equals(ProductTypeEnum.TRAINING)){
            trainingService.delete(productId);
        }else{
        }

        merchantProductRepository.deleteByPid(productId);
        return APIResult.ok();
    }

    @Transactional
    public APIResult<Course> create(CourseDTO courseDTO) {
        APIResult<Course> courseAPIResult = courseService.create(courseDTO);
        if (courseAPIResult.getSuccess()) {
            Product product = convert(courseAPIResult.getData());
            productRepository.save(product);
        }

        return courseAPIResult;
    }

    @Transactional
    public APIResult<Support> create(SupportDTO supportDTO) {
        APIResult<Support> supportAPIResult = supportService.create(supportDTO);
        if (supportAPIResult.getSuccess()) {
            Product product = convert(supportAPIResult.getData());
            productRepository.save(product);
        }

        return supportAPIResult;
    }

    @Transactional
    public APIResult<Package> create(PackageDTO packageDTO) {
        APIResult<Package> packageAPIResult = packageService.create(packageDTO);
        if (packageAPIResult.getSuccess()) {
            Product product = convert(packageAPIResult.getData());
            productRepository.save(product);
        }

        return packageAPIResult;
    }

    @Transactional
    public APIResult<Course> update(CourseDTO courseDTO) {
        APIResult<Course> courseAPIResult = courseService.update(courseDTO);
        if (courseAPIResult.getSuccess()) {
            Product product = convert(courseAPIResult.getData());
            productRepository.update(product);
        }

        return courseAPIResult;
    }

    @Transactional
    public APIResult<Training> create(TrainingDTO trainingDTO) {
        APIResult<Training> targetAPIResult = trainingService.create(trainingDTO);
        if (targetAPIResult.getSuccess()) {
            Product product = convert(targetAPIResult.getData());
            productRepository.save(product);
        }

        return targetAPIResult;
    }

    @Transactional
    public APIResult<Support> update(SupportDTO supportDTO) {
        APIResult<Support> supportAPIResult = supportService.update(supportDTO);
        if (supportAPIResult.getSuccess()) {
            Product product = convert(supportAPIResult.getData());
            productRepository.update(product);
        }

        return supportAPIResult;
    }

    @Transactional
    public APIResult<Package> update(PackageDTO packageDTO) {
        APIResult<Package> packageAPIResult = packageService.update(packageDTO);
        if (packageAPIResult.getSuccess()) {
            Product product = convert(packageAPIResult.getData());
            productRepository.update(product);
        }

        return packageAPIResult;
    }

    @Transactional
    public APIResult<Training> update(TrainingDTO trainingDTO) {
        APIResult<Training> targetAPIResult = trainingService.update(trainingDTO);
        if (targetAPIResult.getSuccess()) {
            Product product = convert(targetAPIResult.getData());
            productRepository.update(product);
        }

        return targetAPIResult;
    }

    public APIResult<List<Product>> findByPids(List<String> pidList) {
        List<Product> productList = productRepository.findByPids(pidList);
        return APIResult.ok(productList);
    }

    public Product findProductByPid(String pid) {
        return productRepository.findByPid(pid);
    }
}

